@@ -1,4 +1,5 @@
 """Internally used rule classes."""
+
 from __future__ import annotations
 
 import inspect
@@ -9,9 +10,11 @@ from typing import TYPE_CHECKING, Any
 from ansiblelint.constants import RULE_DOC_URL
 
 if TYPE_CHECKING:
+    from ansiblelint.config import Options
     from ansiblelint.errors import MatchError
     from ansiblelint.file_utils import Lintable
     from ansiblelint.rules import RulesCollection
+    from ansiblelint.utils import Task
 
 _logger = logging.getLogger(__name__)
 LOAD_FAILURE_MD = """\
@@ -43,6 +46,8 @@ class BaseRule:
     link: str = ""
     has_dynamic_tags: bool = False
     needs_raw_task: bool = False
+    # Used to mark rules that we will never unload (internal ones)
+    unloadable: bool = False
     # We use _order to sort rules and to ensure that some run before others,
     # _order 0 for internal rules
     # _order 1 for rules that check that data can be loaded
@@ -87,7 +92,7 @@ class BaseRule:
             for method in [self.matchlines, self.matchtasks, self.matchyaml]:
                 try:
                     matches.extend(method(file))
-                except Exception as exc:  # pylint: disable=broad-except
+                except Exception as exc:  # pylint: disable=broad-except # noqa: BLE001
                     _logger.warning(
                         "Ignored exception from %s.%s while processing %s: %s",
                         self.__class__.__name__,
@@ -95,6 +100,7 @@ class BaseRule:
                         str(file),
                         exc,
                     )
+                    _logger.debug("Ignored exception details", exc_info=True)
         else:
             matches.extend(self.matchdir(file))
         return matches
@@ -104,7 +110,9 @@ class BaseRule:
         return []
 
     def matchtask(
-        self, task: dict[str, Any], file: Lintable | None = None
+        self,
+        task: Task,
+        file: Lintable | None = None,
     ) -> bool | str | MatchError | list[MatchError]:
         """Confirm if current rule is matching a specific task.
 
@@ -146,6 +154,32 @@ class BaseRule:
         """Return a AnsibleLintRule instance representation."""
         return self.id + ": " + self.shortdesc
 
+    @classmethod
+    def ids(cls) -> dict[str, str]:
+        """Return a dictionary ids and their messages.
+
+        This is used by the ``--list-tags`` option to ansible-lint.
+        """
+        return getattr(cls, "_ids", {cls.id: cls.shortdesc})
+
+    @property
+    def rule_config(self) -> dict[str, Any]:
+        """Retrieve rule specific configuration."""
+        rule_config = self.options.rules.get(self.id, {})
+        if not isinstance(rule_config, dict):  # pragma: no branch
+            msg = f"Invalid rule config for {self.id}: {rule_config}"
+            raise RuntimeError(msg)
+        return rule_config
+
+    @property
+    def options(self) -> Options:
+        """Used to access linter configuration."""
+        if self._collection is None:
+            msg = f"A rule ({self.id}) that is not part of a collection cannot access its configuration."
+            _logger.warning(msg)
+            raise RuntimeError(msg)
+        return self._collection.options
+
 
 # pylint: enable=unused-argument
 
@@ -154,10 +188,12 @@ class RuntimeErrorRule(BaseRule):
     """Unexpected internal error."""
 
     id = "internal-error"
+    shortdesc = "Unexpected internal error"
     severity = "VERY_HIGH"
     tags = ["core"]
     version_added = "v5.0.0"
     _order = 0
+    unloadable = True
 
 
 class AnsibleParserErrorRule(BaseRule):
@@ -169,6 +205,7 @@ class AnsibleParserErrorRule(BaseRule):
     tags = ["core"]
     version_added = "v5.0.0"
     _order = 0
+    unloadable = True
 
 
 class LoadingFailureRule(BaseRule):
@@ -179,8 +216,12 @@ class LoadingFailureRule(BaseRule):
     severity = "VERY_HIGH"
     tags = ["core", "unskippable"]
     version_added = "v4.3.0"
-    help = LOAD_FAILURE_MD
+    _help = LOAD_FAILURE_MD
     _order = 0
+    _ids = {
+        "load-failure[not-found]": "File not found",
+    }
+    unloadable = True
 
 
 class WarningRule(BaseRule):
@@ -192,3 +233,4 @@ class WarningRule(BaseRule):
     tags = ["core", "experimental"]
     version_added = "v6.8.0"
     _order = 0
+    unloadable = True
