@@ -72,13 +72,14 @@ class CommandsInsteadOfModulesRule(AnsibleLintRule):
         "git": ["branch", "log", "lfs", "rev-parse"],
         "systemctl": [
             "--version",
+            "get-default",
             "kill",
             "set-default",
             "set-property",
             "show-environment",
             "status",
         ],
-        "yum": ["clean"],
+        "yum": ["clean", "history", "info"],
         "rpm": ["--nodeps"],
     }
 
